# Natural Language Toolkit: HuggingFace dataset integration
#
# Copyright (C) 2001-2026 NLTK Project
# URL: <https://www.nltk.org/>
# For license information, see LICENSE.TXT

from nltk.huggingface.dataset import download

__all__ = ["download"]
